function Board = initboard(BoardSize,InitXLoc,InitYLoc,TargetXLoc,TargetYLoc)

global Q BoardColor BoardText statusText turnText whiteText blackText XLine YLine

Acolor = ['w';'b'];

% define the board
FigSize = 320;
ButtonWidth = 70;

if length(BoardSize) > 1, 
    Board = BoardSize; 
    BoardSize = size(Board,1);
    [i,j] = find(Board==1);
    if ~isempty(i),
        Board(i,j) = 0;
    end;        
else    
    Board = zeros(BoardSize,BoardSize);
end;    

if nargin < 2,
    InitXLoc = 1; InitYLoc = 1;
end;    
if nargin == 4,
    Board(TargetXLoc,TargetYLoc) = 3;
end;

Board(InitXLoc,InitYLoc) = 1;

% set up the main frame
fig1 = figure(1); clf; 
set(fig1, 'Position', [407 241 FigSize+ButtonWidth FigSize],...
   'Name','Grid World',...
   'NumberTitle','off');,...

% user control panel
left = 400;
bottom = 300;
width = 75;
height = 30;
ymod = 30;

uicontrol('Style','Text',...
   'Position',[(FigSize+ButtonWidth-300)/2 5 300 25],...
   'BackgroundColor',[0.8 0.8 0.8],...
   'String','Left click ... obstacle, right click .. goal');

whiteText = uicontrol('Style','Text',...
   'Position',[left bottom-7*ymod width height],...
      'BackgroundColor',[0.8 0.8 0.8],...
      'String',['x = ' num2str(InitXLoc)]);
   
blackText = uicontrol('Style','Text',...
   'Position',[left bottom-8*ymod width height],...
      'BackgroundColor',[0.8 0.8 0.8],...
      'String',['y = ' num2str(InitYLoc)]);

% creating the board grids
XLine = linspace(0,FigSize,BoardSize+1);
YLine = linspace(0,FigSize,BoardSize+1);

for i=1:(BoardSize+1)
   line([XLine(i) XLine(i)], [YLine(1) YLine(BoardSize+1)],'Color',[0 0 0]);
   line([XLine(1) XLine(BoardSize+1)], [YLine(i) YLine(i)],'Color',[0 0 0]);
end

axis([0,FigSize+ButtonWidth,0,FigSize]);
hold on; axis off;

% determine the radius of the piece
gap = (XLine(2)-XLine(1))/2;
radius = gap - (gap*0.25); % leave a 25% side margin

% save the center of the pieces for drawing purposes
XCenter = linspace((XLine(2)-XLine(1))/2, FigSize-(XLine(2)-XLine(1))/2, BoardSize);
YCenter = fliplr(linspace((YLine(2)-YLine(1))/2, FigSize-(YLine(2)-YLine(1))/2, BoardSize));
BoardCenterX = repmat(XCenter,BoardSize,1);
BoardCenterY = repmat((FigSize-YCenter)',1,BoardSize);

Board = Board';

for i=1:BoardSize,
   for j=1:BoardSize,
      X = BoardCenterX(i,j) + radius*cos(0:1/100:2*pi);
      Y = BoardCenterY(i,j) + radius*sin(0:1/100:2*pi);
      if isnan(Board(i,j)), 
            BoardColor(i,j) = fill(X,Y,[.6 .6 .6]); 
      else
        if Board(i,j) > 0,
                BoardColor(i,j) = fill(X,Y,Acolor(abs(Board(i,j))));
        elseif Board(i,j) < 0,
            BoardColor(i,j) = fill(X,Y,'r');
            set(BoardColor(i,j),'EdgeColor',Acolor(abs(Board(i,j))));
        else
         	BoardColor(i,j) = fill(X,Y,[0.8 0.8 0.8]);
            set(BoardColor(i,j),'EdgeColor',[0.8 0.8 0.8]);
        end;                
      end
      BoardText(i,j) = text(BoardCenterX(i,j)+12,BoardCenterY(i,j),'');
      set(BoardText(i,j),'horizontalalignment','right');
  end
end

for i=1:BoardSize,
   for j=1:BoardSize,
      b = -inf;
      if i+1 <= BoardSize, b = max(b,Q(i+1,j,3)); end;    
      if j+1 <= BoardSize, b = max(b,Q(i,j+1,4)); end;    
      if i-1 >= 1, b = max(b,Q(i-1,j,1)); end;    
      if j-1 >= 1, b = max(b,Q(i,j-1,2)); end;
      b = sprintf('%1.0f',b);
      set(BoardText(j,i),'String',b); 
  end;
end;

if 0,
for i=1:BoardSize,
   for j=1:BoardSize,
      if i+1 <= BoardSize, set(BoardText(j,i+1),'String',num2str(Q(i,j,1))); end;    
      if j+1 <= BoardSize, set(BoardText(j+1,i),'String',num2str(Q(i,j,2))); end;    
      if i-1 >= 1, set(BoardText(j,i-1),'String',num2str(Q(i,j,3))); end;    
      if j-1 >= 1, set(BoardText(j-1,i),'String',num2str(Q(i,j,4))); end;    
  end;
end;
end;

Board = Board';
% end of function %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%